@@ -17,6 +17,17 @@ from .config import (
 
 CLI_TOOL = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'commands', 'cli_main.py')
 
+def is_pypi_installation() -> bool:
+    """
+    Detect if this is a PyPI installation (site-packages) vs development installation.
+
+    Returns:
+        True if installed via pip (in site-packages), False if running from source
+    """
+    cli_tool_path = Path(CLI_TOOL).resolve()
+    # Check if the path contains 'site-packages' - indicates PyPI installation
+    return 'site-packages' in str(cli_tool_path) or 'dist-packages' in str(cli_tool_path)
+
 def get_company_short(company_id: str) -> str:
     """
     Get shortened company ID for runtime paths to avoid socket path length issues.
@@ -39,15 +50,15 @@ def _track_ssh_operation(operation: str, host: str = "unknown", success: bool =
         pass
 
 def get_cli_command() -> list:
-    if not is_windows(): return [CLI_TOOL]
-    # On Windows, try 'python' first since 'python3' usually doesn't exist
-    for cmd in ['python', 'python3', 'py']:
-        try:
-            result = subprocess.run([cmd, '--version'], capture_output=True, text=True, timeout=5)
-            if result.returncode == 0 and 'Python 3' in result.stdout: 
-                return [cmd, CLI_TOOL]
-        except: continue
-    return ['python', CLI_TOOL]
+    """Get the command to run CLI operations as subprocess.
+
+    Always uses the current Python interpreter (sys.executable) for maximum compatibility
+    across all installation methods (PyPI, development, venvs, etc.) and platforms.
+
+    PyPI-installed Python files don't have execute permissions, so we must use the
+    Python interpreter explicitly rather than trying to execute the .py file directly.
+    """
+    return [sys.executable, CLI_TOOL]
 
 def is_windows() -> bool:
     return platform.system().lower() == 'windows'
@@ -431,6 +442,31 @@ def _decode_ssh_key(ssh_key: str) -> str:
 
     return ssh_key
 
+def _decode_host_entry(host_entry: str) -> str:
+    """Decode and normalize host entry for known_hosts file"""
+    import base64
+
+    if not host_entry:
+        return host_entry
+
+    # Decode base64 if needed
+    # Known_hosts entries typically contain spaces (e.g., "hostname ssh-rsa AAAA...")
+    # If there are no spaces and no newlines, it's likely base64 encoded
+    if ' ' not in host_entry and '\n' not in host_entry:
+        try:
+            host_entry = base64.b64decode(host_entry).decode('utf-8')
+        except Exception:
+            # If decoding fails, use as-is (might already be decoded)
+            pass
+
+    # Normalize line endings to Unix format
+    host_entry = host_entry.replace('\r\n', '\n').replace('\r', '\n')
+
+    # Remove trailing newlines (we'll add one when writing to file)
+    host_entry = host_entry.rstrip('\n')
+
+    return host_entry
+
 def _convert_path_for_ssh(path: str, ssh_executable: str = None) -> str:
     """Convert Windows paths for SSH compatibility based on SSH implementation"""
     if not path or not is_windows():
@@ -462,29 +498,36 @@ def _convert_path_for_ssh(path: str, ssh_executable: str = None) -> str:
 
     return path
 
-def _setup_ssh_options(host_entry: str, known_hosts_path: str, key_path: str = None, ssh_executable: str = None) -> str:
-    """Setup SSH options with proper path handling for cross-platform compatibility"""
+def _setup_ssh_options(host_entry: str, known_hosts_path: str, key_path: str = None, ssh_executable: str = None, port: int = 22) -> str:
+    """Setup SSH options with strict host key verification
+
+    Args:
+        host_entry: Expected host key entry from vault (REQUIRED)
+        known_hosts_path: Path to known_hosts file
+        key_path: Optional path to SSH private key file
+        ssh_executable: Optional SSH executable path for Windows compatibility
+        port: SSH port number (default: 22)
+
+    Raises:
+        ValueError: If host_entry is None or empty (no insecure connections allowed)
+    """
+    # Security: ALWAYS require host key from service - no exceptions
+    if not host_entry:
+        raise ValueError(
+            "Security Error: No host key found in vault. "
+            "The service MUST provide a host key for all SSH connections. "
+            "Contact your administrator to add the host key to the machine vault."
+        )
+
     # Convert paths based on SSH implementation (MSYS2 vs Windows OpenSSH)
     if known_hosts_path:
         known_hosts_path = _convert_path_for_ssh(known_hosts_path, ssh_executable)
     if key_path:
         key_path = _convert_path_for_ssh(key_path, ssh_executable)
 
-    # Security: Use different strategies based on host_entry availability
-    if host_entry:
-        # Host entry exists in vault - use strict checking to prevent MITM attacks
-        # This is the secure path for machines with known host keys
-        base_opts = f"-o StrictHostKeyChecking=yes -o UserKnownHostsFile={known_hosts_path}"
-        _track_ssh_operation("host_key_verification", "known_host", True)
-    else:
-        # No host entry - this is a first-time connection or dev mode
-        # For security, we still want to save the host key for future verification
-        # but we need to accept it initially to establish the connection
-
-        # Use a temporary known_hosts file to capture the host key
-        # Always use the provided known_hosts path (never null device)
-        base_opts = f"-o StrictHostKeyChecking=accept-new -o UserKnownHostsFile={known_hosts_path}"
-        _track_ssh_operation("host_key_verification", "new_host", True)
+    # STRICT host key checking - we trust ONLY what the service provides
+    base_opts = f"-o StrictHostKeyChecking=yes -o UserKnownHostsFile={known_hosts_path} -p {port}"
+    _track_ssh_operation("host_key_verification", "known_host", True)
 
     # Add additional security options
     security_opts = "-o PasswordAuthentication=no -o PubkeyAuthentication=yes -o PreferredAuthentications=publickey"
@@ -494,9 +537,19 @@ def _setup_ssh_options(host_entry: str, known_hosts_path: str, key_path: str = N
 
     return f"{all_opts} -i {key_path}" if key_path else all_opts
 
-def setup_ssh_agent_connection(ssh_key: str, host_entry: str = None) -> Tuple[str, str, str]:
+def setup_ssh_agent_connection(ssh_key: str, host_entry: str, port: int = 22) -> Tuple[str, str, str]:
+    """Setup SSH connection using ssh-agent with strict host key verification
+
+    Args:
+        ssh_key: SSH private key content
+        host_entry: Expected host key from vault (REQUIRED)
+        port: SSH port number (default: 22)
+
+    Raises:
+        ValueError: If host_entry is None or empty
+    """
     import subprocess
-    
+
     ssh_key = _decode_ssh_key(ssh_key)
     
     try:
@@ -525,17 +578,32 @@ def setup_ssh_agent_connection(ssh_key: str, host_entry: str = None) -> Tuple[st
     except Exception as e:
         raise RuntimeError(f"SSH agent setup failed: {e}")
     
-    known_hosts_file_path = None
+    # Always create a known_hosts file, even for first-time connections
+    # This allows SSH to save the host key for future verification
+    known_hosts_file_path = create_temp_file(suffix='_known_hosts', prefix='known_hosts_')
+
     if host_entry:
-        known_hosts_file_path = create_temp_file(suffix='_known_hosts', prefix='known_hosts_')
+        # Decode and write the existing host entry from the vault
+        host_entry = _decode_host_entry(host_entry)
         with open(known_hosts_file_path, 'w') as f: f.write(host_entry + '\n')
-    
-    ssh_opts = _setup_ssh_options(host_entry, known_hosts_file_path)
-    
+    # If no host_entry, the file is empty but will be used to store the new host key
+
+    ssh_opts = _setup_ssh_options(host_entry, known_hosts_file_path, port=port)
+
     return ssh_opts, agent_pid, known_hosts_file_path
 
-def setup_ssh_for_connection(ssh_key: str, host_entry: str = None, ssh_executable: str = None) -> Tuple[str, str, str]:
-    """Setup SSH connection with enhanced error handling and cross-platform compatibility"""
+def setup_ssh_for_connection(ssh_key: str, host_entry: str, ssh_executable: str = None, port: int = 22) -> Tuple[str, str, str]:
+    """Setup SSH connection with strict host key verification
+
+    Args:
+        ssh_key: SSH private key content
+        host_entry: Expected host key from vault (REQUIRED)
+        ssh_executable: Optional SSH executable path for Windows compatibility
+        port: SSH port number (default: 22)
+
+    Raises:
+        ValueError: If host_entry is None or empty
+    """
     try:
         ssh_key = _decode_ssh_key(ssh_key)
     except ValueError as e:
@@ -580,13 +648,14 @@ def setup_ssh_for_connection(ssh_key: str, host_entry: str = None, ssh_executabl
     # This allows SSH to save the host key for future verification
     known_hosts_file_path = create_temp_file(suffix='_known_hosts', prefix='known_hosts_')
     if host_entry:
-        # Write the existing host entry from the vault
-        with open(known_hosts_file_path, 'w') as f: 
+        # Decode and write the existing host entry from the vault
+        host_entry = _decode_host_entry(host_entry)
+        with open(known_hosts_file_path, 'w') as f:
             f.write(host_entry + '\n')
     # If no host_entry, the file is empty but will be used to store the new host key
-    
-    ssh_opts = _setup_ssh_options(host_entry, known_hosts_file_path, ssh_key_file_path, ssh_executable)
-    
+
+    ssh_opts = _setup_ssh_options(host_entry, known_hosts_file_path, ssh_key_file_path, ssh_executable, port)
+
     return ssh_opts, ssh_key_file_path, known_hosts_file_path
 
 def cleanup_ssh_agent(agent_pid: str, known_hosts_file: str = None):
@@ -601,22 +670,33 @@ def cleanup_ssh_key(ssh_key_file: str, known_hosts_file: str = None):
         if file_path and os.path.exists(file_path): os.unlink(file_path)
 
 class SSHConnection:
-    """Context manager for SSH connections with automatic cleanup.
-    
+    """Context manager for SSH connections with strict security and automatic cleanup.
+
+    Requires host key from service for all connections - no insecure connections allowed.
     Tries SSH agent first, falls back to file-based keys if agent fails.
     Automatically cleans up resources on exit.
     """
-    
-    def __init__(self, ssh_key: str, host_entry: str = None, prefer_agent: bool = True):
+
+    def __init__(self, ssh_key: str, host_entry: str, port: int = 22, prefer_agent: bool = True):
         """Initialize SSH connection context.
-        
+
         Args:
             ssh_key: SSH private key content
-            host_entry: Optional known_hosts entry
+            host_entry: Host key from vault (REQUIRED for security)
+            port: SSH port number (default: 22)
             prefer_agent: Whether to try SSH agent first (default: True)
+
+        Raises:
+            ValueError: If host_entry is None or empty
         """
+        if not host_entry:
+            raise ValueError(
+                "Security Error: host_entry is required for SSH connections. "
+                "The service must provide a host key from the vault."
+            )
         self.ssh_key = ssh_key
         self.host_entry = host_entry
+        self.port = port
         self.prefer_agent = prefer_agent
         self.ssh_opts = None
         self.agent_pid = None
@@ -634,7 +714,7 @@ class SSHConnection:
             if self.prefer_agent:
                 try:
                     self.ssh_opts, self.agent_pid, self.known_hosts_file = setup_ssh_agent_connection(
-                        self.ssh_key, self.host_entry
+                        self.ssh_key, self.host_entry, self.port
                     )
                     self._using_agent = True
                     success = True
@@ -649,7 +729,7 @@ class SSHConnection:
 
             # File-based fallback
             self.ssh_opts, self.ssh_key_file, self.known_hosts_file = setup_ssh_for_connection(
-                self.ssh_key, self.host_entry
+                self.ssh_key, self.host_entry, port=self.port
             )
             success = True
             _track_ssh_operation("connection_setup", "file-based", True,
@@ -689,13 +769,19 @@ class SSHConnection:
 
 class SSHTunnelConnection(SSHConnection):
     """Context manager for SSH connections that need to maintain tunnels.
-    
+
     This is a special variant that doesn't automatically cleanup SSH resources
     on exit, allowing tunnels to persist. Cleanup must be done manually.
     """
-    
-    def __init__(self, ssh_key: str, host_entry: str = None, prefer_agent: bool = True):
-        """Initialize SSH tunnel connection context."""
+
+    def __init__(self, ssh_key: str, host_entry: str, prefer_agent: bool = True):
+        """Initialize SSH tunnel connection context.
+
+        Args:
+            ssh_key: SSH private key content
+            host_entry: Host key from vault (REQUIRED for security)
+            prefer_agent: Whether to try SSH agent first (default: True)
+        """
         super().__init__(ssh_key, host_entry, prefer_agent)
         self._cleanup_on_exit = True
     
@@ -731,16 +817,18 @@ def get_machine_connection_info(machine_info: Dict[str, Any]) -> Dict[str, Any]:
                 except json.JSONDecodeError as e:
                     print(colorize(f"Failed to parse vaultContent: {e}", 'RED'))
 
-    ip = vault.get('ip') or vault.get('IP')
-    ssh_user = vault.get('user') or vault.get('USER')
-    datastore = vault.get('datastore') or vault.get('DATASTORE')
-    host_entry = vault.get('hostEntry') or vault.get('HOST_ENTRY')
+    ip = vault.get('ip')
+    ssh_user = vault.get('user')
+    datastore = vault.get('datastore')
+    host_entry = vault.get('host_entry')
+    port = vault.get('port', 22)  # Default to port 22 if not specified
 
     # DEBUG: Log machine vault info
     logger.debug(f"[get_machine_connection_info] Machine '{machine_name}' vault:")
     logger.debug(f"  - ip: {ip}")
     logger.debug(f"  - user: {ssh_user}")
     logger.debug(f"  - datastore: {datastore}")
+    logger.debug(f"  - port: {port}")
 
     # Validate required fields
     if not datastore:
@@ -774,6 +862,7 @@ def get_machine_connection_info(machine_info: Dict[str, Any]) -> Dict[str, Any]:
     return {
         'ip': ip,
         'user': ssh_user,
+        'port': port,
         'universal_user': universal_user,
         'universal_user_id': universal_user_id,
         'datastore': datastore,
@@ -854,9 +943,11 @@ def initialize_cli_command(args, parser, requires_cli_tool=True):
     
     # Validate CLI tool if required
     if requires_cli_tool:
-        if not os.path.exists(CLI_TOOL): 
+        if not os.path.exists(CLI_TOOL):
             error_exit(f"rediacc not found at {CLI_TOOL}")
-        if not is_windows() and not os.access(CLI_TOOL, os.X_OK): 
+        # Only check executable permissions for development installations
+        # PyPI installations don't need cli_main.py to be executable (entry points handle execution)
+        if not is_windows() and not is_pypi_installation() and not os.access(CLI_TOOL, os.X_OK):
             error_exit(f"rediacc is not executable at {CLI_TOOL}")
 
 def add_common_arguments(parser, include_args=None, required_overrides=None):
@@ -967,18 +1058,19 @@ def test_ssh_connectivity(ip: str, port: int = 22, timeout: int = 5) -> Tuple[bo
 
     return result
 
-def validate_machine_accessibility(machine_name: str, team_name: str, ip: str, repo_name: str = None):
-    print(f"Testing connectivity to {ip}...")
-    is_accessible, error_msg = test_ssh_connectivity(ip)
+def validate_machine_accessibility(machine_name: str, team_name: str, ip: str, port: int = 22, repo_name: str = None):
+    print(f"Testing connectivity to {ip}:{port}...")
+    is_accessible, error_msg = test_ssh_connectivity(ip, port)
     if is_accessible: print(colorize("✓ Machine is accessible", 'GREEN')); return
-    
+
     print(colorize(f"\n✗ Machine '{machine_name}' is not accessible", 'RED'))
     print(colorize(f"  Error: {error_msg}", 'RED'))
     print(colorize("\nPossible reasons:", 'YELLOW'))
-    for reason in ["The machine is offline or powered down", "Network connectivity issues between client and machine", "Firewall blocking SSH port (22)", "Incorrect IP address in machine configuration"]:
+    for reason in ["The machine is offline or powered down", "Network connectivity issues between client and machine", f"Firewall blocking SSH port ({port})", "Incorrect IP address in machine configuration"]:
         print(colorize(f"  • {reason}", 'YELLOW'))
-    
+
     print(colorize(f"\nMachine IP: {ip}", 'BLUE'))
+    print(colorize(f"Port: {port}", 'BLUE'))
     print(colorize(f"Team: {team_name}", 'BLUE'))
     if repo_name:
         print(colorize(f"Repository: {repo_name}", 'BLUE'))
@@ -1094,15 +1186,16 @@ class RepositoryConnection:
     
     def ssh_context(self, prefer_agent: bool = True):
         """Get SSH connection context manager.
-        
+
         Args:
             prefer_agent: Whether to try SSH agent first (default: True)
-            
+
         Returns:
             SSHConnection context manager
         """
         host_entry = self._connection_info.get('host_entry')
-        return SSHConnection(self._ssh_key, host_entry, prefer_agent)
+        port = self._connection_info.get('port', 22)
+        return SSHConnection(self._ssh_key, host_entry, port, prefer_agent)
     
     @property
     def ssh_destination(self) -> str:
